# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('image_collection', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='imageslide',
            name='caption_headline',
            field=models.CharField(help_text='This text is displayed as title of the image.', max_length=256, verbose_name='caption', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='imageslide',
            name='caption',
            field=models.CharField(help_text='This text is displayed as description of the image.', max_length=512, verbose_name='caption', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='imageslide',
            name='link',
            field=models.URLField(help_text='Enter URL, that the image should link to.', verbose_name='link'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import image_collection.models


class Migration(migrations.Migration):

    dependencies = [
        ('image_collection', '0003_auto_20160113_0445'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='imageslide',
            name='link',
        ),
        migrations.AddField(
            model_name='imageslide',
            name='external_link',
            field=models.URLField(help_text='E.g. "http://www.example.com/my-page/". Enter absolute URL, that the image should link to.', verbose_name='external link', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='imageslide',
            name='internal_link',
            field=image_collection.models.RelativeURLField(help_text='E.g. "/my-page/". Enter slug of internal pager, that the image should link to.', verbose_name='internal link', blank=True),
            preserve_default=True,
        ),
    ]



"""Templatetags for the image_collection app."""
from django import template

from ..models import ImageCollection


register = template.Library()


@register.inclusion_tag(takes_context=True,
                        file_name='image_collection/tags/bs3_carousel.html')
def render_bs3_carousel(context, identifier):
    """Renders an image collection as Bootstrap 3 carousel."""
    try:
        collection = ImageCollection.objects.get(identifier=identifier)
    except ImageCollection.DoesNotExist:
        collection = None
    context.update({'collection': collection})
    return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ImageCollection',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=512, verbose_name='name')),
                ('identifier', models.SlugField(unique=True, max_length=512, verbose_name='identifier')),
            ],
            options={
                'ordering': ('name', 'identifier'),
                'verbose_name': ('image collection',),
                'verbose_name_plural': ('image collections',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ImageSlide',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('image', models.ImageField(upload_to=b'image_slides', verbose_name='image')),
                ('caption', models.CharField(help_text='This text is displayed on top of the image or as its description.', max_length=256, verbose_name='caption', blank=True)),
                ('alt_text', models.CharField(help_text="This will go into the ``alt`` attribute of the image's HTML markup.", max_length=128, verbose_name='alt text', blank=True)),
                ('link', models.URLField(help_text='Enter URL, that the image should link to. (not required)', verbose_name='link')),
                ('start_date', models.DateTimeField(null=True, verbose_name='publish date', blank=True)),
                ('end_date', models.DateTimeField(null=True, verbose_name='unpublish date', blank=True)),
                ('collection', models.ForeignKey(related_name='images', verbose_name='image collection', to='image_collection.ImageCollection')),
            ],
            options={
                'ordering': ('collection', 'start_date', 'end_date', 'alt_text'),
                'verbose_name': 'image',
                'verbose_name_plural': 'images',
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-10 00:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('image_collection', '0004_auto_20160113_0458'),
    ]

    operations = [
        migrations.AddField(
            model_name='imageslide',
            name='image_mobile',
            field=models.ImageField(blank=True, null=True, upload_to=b'image_slides', verbose_name='image_mobile'),
        ),
    ]



"""Models for the image_collection app."""
import re
from django import forms
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

relative_url_re = re.compile(r'^[-a-zA-Z0-9_\/]+$')
validate_relative_url = RegexValidator(
    relative_url_re,
    _("Enter a valid 'relative URL' consisting of letters, numbers,"
      " underscores, forward slashes or hyphens."), 'invalid')


class RelativeURLFormField(forms.CharField):
    default_validators = [validate_relative_url]

    def clean(self, value):
        value = self.to_python(value).strip()
        return super(RelativeURLFormField, self).clean(value)


class RelativeURLField(models.fields.CharField):
    default_validators = [validate_relative_url]
    description = _("Relative URL (up to %(max_length)s)")

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 50)
        # Set db_index=True unless it's been set manually.
        if 'db_index' not in kwargs:
            kwargs['db_index'] = True
        super(RelativeURLField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(RelativeURLField, self).deconstruct()
        if kwargs.get("max_length", None) == 50:
            del kwargs['max_length']
        if self.db_index is False:
            kwargs['db_index'] = False
        else:
            del kwargs['db_index']
        return name, path, args, kwargs

    def get_internal_type(self):
        return "CharField"

    def formfield(self, **kwargs):
        defaults = {'form_class': RelativeURLFormField}
        defaults.update(kwargs)
        return super(RelativeURLField, self).formfield(**defaults)


@python_2_unicode_compatible
class ImageCollection(models.Model):
    """
    This model wraps together a collection of images.

    :name: Human readable title for this collection
    :identifier: The unique identifier for this collection

    """

    class Meta:
        ordering = ('name', 'identifier')
        verbose_name = _('image collection')
        verbose_name_plural = _('image collections')

    name = models.CharField(
        max_length=512,
        verbose_name=_('name'),
    )

    identifier = models.SlugField(
        unique=True,
        max_length=512,
        verbose_name=_('identifier'),
    )

    def __str__(self):  # pragma: no cover
        return '{0} ({1}): {2}'.format(
            self._meta.verbose_name.title(), self.pk, self.identifier)

    def get_published_images(self):
        return self.images.filter(
            # TODO is this too complicated?
            models.Q(
                start_date__lte=now(),
                end_date__isnull=True,
            ) | models.Q(
                start_date__isnull=True,
                end_date__gte=now(),
            ) | models.Q(
                start_date__isnull=True,
                end_date__isnull=True,
            ) | models.Q(
                start_date__lte=now(),
                end_date__gte=now(),
            )
        ).distinct()

    def is_public(self):
        """Returns True, if any image exists, that is currently published."""
        return self.get_published_images().exists()


@python_2_unicode_compatible
class ImageSlide(models.Model):
    """
    One image object and its extra meta information.

    :collection: The collection this image belongs to
    :image: The image file
    :image_mobile: Optional alternative image to be used for mobile devices
    :alt_text: What goes into the alt attribute
    :link: If the image should link to any specific URL, put it here. Best is
      to use the ``url`` property instead of this, since that falls back to the
      image url automatically.
    :data-class: Optional. If you want to attach click events to this image,
      fill out this field.
    :start_date: The datetime, where the image should start to be published
    :start_end: The datetime, where the image should no longer be published

    """

    class Meta:
        ordering = ('collection', 'start_date', 'end_date', 'alt_text')
        verbose_name = _('image')
        verbose_name_plural = _('images')

    collection = models.ForeignKey(
        ImageCollection,
        verbose_name=_('image collection'),
        related_name='images',
    )

    image = models.ImageField(
        upload_to='image_slides',
        verbose_name=_('image'),
    )

    image_mobile = models.ImageField(
        upload_to='image_slides',
        verbose_name=_('image_mobile'),
        blank=True,
        null=True,
    )

    caption_headline = models.CharField(
        max_length=256,
        verbose_name=_('caption headline'),
        help_text=_('This text is displayed as title of the image.'),
        blank=True,
    )

    caption = models.CharField(
        max_length=512,
        verbose_name=_('caption'),
        help_text=_('This text is displayed as description of the image.'),
        blank=True,
    )

    alt_text = models.CharField(
        max_length=128,
        verbose_name=_('alt text'),
        help_text=_('This will go into the ``alt`` attribute of the image\'s'
                    ' HTML markup.'),
        blank=True,
    )

    external_link = models.URLField(
        verbose_name=_('external link'),
        help_text=_('E.g. "http://www.example.com/my-page/". Enter absolute'
                    ' URL, that the image should link to.'),
        blank=True,
    )

    internal_link = RelativeURLField(
        verbose_name=_('internal link'),
        help_text=_('E.g. "/my-page/". Enter slug of internal pager, that the'
                    ' image should link to.'),
        blank=True,
    )

    data_class = models.CharField(
        verbose_name=_('data class'),
        max_length='512',
        help_text=(
            'Set this if you want to handle click events on this image via'
            ' JavaScript.'),
        blank=True
    )

    start_date = models.DateTimeField(
        verbose_name=_('publish date'),
        blank=True, null=True,
    )

    end_date = models.DateTimeField(
        verbose_name=_('unpublish date'),
        blank=True, null=True,
    )

    def __str__(self):  # pragma: no cover
        return '{0} ({1}): {2}'.format(
            self._meta.verbose_name.title(), self.pk, self.alt_text)

    def clean(self):
        if self.internal_link and self.external_link:
            raise ValidationError(_('You cannot have both and internal and an'
                                    ' external link. Please remove at least'
                                    ' one.'))

    @property
    def url(self):  # pragma: no cover
        """Should always return a proper url."""
        return self.external_link.strip() or self.internal_link.strip() \
            or self.image.url

    @url.setter
    def url(self, value):  # pragma: no cover
        self.link = value

    @url.deleter
    def url(self):  # pragma: no cover
        self.link = ''



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('image_collection', '0005_imageslide_image_mobile'),
    ]

    operations = [
        migrations.AddField(
            model_name='imageslide',
            name='data_class',
            field=models.CharField(help_text=b'Set this if you want to handle click events on this image via JavaScript.', max_length=b'512', verbose_name='data class', blank=True),
        ),
    ]



"""Tests for the ``image_collection_tags`` template tags."""
from django.test import TestCase
from mixer.backend.django import mixer
from ..templatetags.image_collection_tags import render_bs3_carousel


class RenderBS3CarouselTestCase(TestCase):
    """Test case for the ``render_bs3_carousel`` template tag."""

    def test_context(self):
        """
        Test if the context of ``render_bs3_carousel`` is set up correctly.

        """
        self.assertIsNone(
            render_bs3_carousel({}, 'foobar')['collection'],
            msg=(
                'When there is no collection with the given identifier, the'
                ' collection inside the context should be "None".'
            ))
        collection = mixer.blend(
            'image_collection.ImageCollection',
            identifier='foobar')
        self.assertEqual(
            render_bs3_carousel({}, 'foobar')['collection'],
            collection,
            msg=(
                'When there is a collection with the given identifier, it'
                ' should appear in the context.'
            )
        )



"""Admin classes for the image_collection app."""
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from . import models


class ImageCollectionAdmin(admin.ModelAdmin):
    """Custom admin for the ``ImageCollection`` model."""
    list_display = ('identifier', 'name')


class ImageSlideAdmin(admin.ModelAdmin):
    """Custom admin for the ``ImageSlide`` model."""
    list_display = (
        'collection', 'get_headline', 'get_caption', 'image', 'image_mobile')

    def get_caption(self, obj):
        return obj.caption
    get_caption.short_description = _('description')

    def get_headline(self, obj):
        return obj.caption_headline
    get_headline.short_description = _('headline')


admin.site.register(models.ImageSlide, ImageSlideAdmin)
admin.site.register(models.ImageCollection, ImageCollectionAdmin)



"""Tests for the models of the image_collection app."""
from django.test import TestCase
from django.utils.timezone import now, timedelta

from mixer.backend.django import mixer


class ImageCollectionTestCase(TestCase):
    """Tests for the ``ImageCollection`` model class."""
    longMessage = True

    def test_instantiation(self):
        image_collection = mixer.blend('image_collection.ImageCollection')
        self.assertTrue(image_collection.pk)

    def test_is_public(self):
        collection = mixer.blend('image_collection.ImageCollection')
        self.assertFalse(collection.is_public(), msg=(
            'Should return False, since there are no images in this'
            ' collection.'
        ))
        mixer.blend(
            'image_collection.ImageSlide',
            collection=collection,
            start_date=now() + timedelta(days=10),
        )
        self.assertFalse(collection.is_public(), msg=(
            'Should return False, since there is no published image.'
        ))
        mixer.blend(
            'image_collection.ImageSlide',
            collection=collection,
            end_date=now() - timedelta(days=10),
        )
        self.assertFalse(collection.is_public(), msg=(
            'Should return False, since there is no published image.'
        ))
        mixer.blend(
            'image_collection.ImageSlide',
            collection=collection,
            start_date=now() - timedelta(days=10),
            end_date=now() + timedelta(days=10),
        )
        self.assertTrue(collection.is_public(), msg=(
            'Should return True, since there is a published image in this'
            ' collection.'
        ))


class ImageSlideTestCase(TestCase):
    """Tests for the ``ImageSlide`` model class."""
    longMessage = True

    def test_instantiation(self):
        image_slide = mixer.blend('image_collection.ImageSlide')
        self.assertTrue(image_slide.pk)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('image_collection', '0002_auto_20160113_0239'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='imagecollection',
            options={'ordering': ('name', 'identifier'), 'verbose_name': 'image collection', 'verbose_name_plural': 'image collections'},
        ),
        migrations.AlterField(
            model_name='imageslide',
            name='caption_headline',
            field=models.CharField(help_text='This text is displayed as title of the image.', max_length=256, verbose_name='caption headline', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='imageslide',
            name='link',
            field=models.URLField(help_text='Enter URL, that the image should link to.', verbose_name='link', blank=True),
            preserve_default=True,
        ),
    ]
